#include <assert.h>
#include <dirent.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <badpenguin-chain.h>
#include <badpenguin-fs.h>
#include "badpenguin-hardware.h"
#include "proc-pci.h"


unsigned int getword(unsigned char *base) {
  return base[0] + ( base[1] * 256 );
}

unsigned long getdword(unsigned char *base) {
  return base[0] + ( base[1] * 256 )+ ( base[2] * 256 * 256 ) + ( base[3] * 256 * 256 * 256 );
}


int query_pci_devices(TComputer *pc, int loiVerbose) {
  char laPciDir[256];
  char laTempDir[256];
  char laTempFile[256];
  char buffer[256];
  DIR *dir1 = NULL;
  DIR *dir2 = NULL;
  struct dirent *lpdirent1 = NULL;
  struct dirent *lpdirent2 = NULL;
  TChainItem    *pciitem = NULL;
  TPciDevice    *mydevice = NULL;
  FILE          *fd = NULL;

  // create pci chain
  if (pc->pci == NULL) {
    pc->pci = chain_create();
  }
  
  // Open main proc dir
  strcpy( laPciDir, PCI_ROOTDIR);
  // Search for XX bus subdirs
  dir1 = opendir(laPciDir);
  assert( dir1 != NULL );
  lpdirent1 = readdir(dir1);
  while (lpdirent1 != NULL) {    
    // Skip dot files
    if (lpdirent1->d_name[0] == '.') {
      lpdirent1 = readdir(dir1);
      continue;
    }
    // Check directory
    if ( 0 != is_directory(laPciDir,lpdirent1->d_name)) {
      lpdirent1 = readdir(dir1);
      continue;
    }    
    // Open the 00 subdirectory    
    if (loiVerbose>1) printf("Found dir: %s \n", lpdirent1->d_name );
    strcpy( laTempDir, laPciDir );
    strcat( laTempDir, "/" );
    strcat( laTempDir, lpdirent1->d_name );
    //    
    dir2 = opendir(laTempDir);
    if (dir2==NULL) {
        lpdirent1 = readdir(dir1);
        continue;    
    }
    assert( dir2 != NULL );
    lpdirent2 = readdir(dir2);
    while (lpdirent2 != NULL) {
      // Skip dot files
      if (lpdirent2->d_name[0] == '.') {
        lpdirent2 = readdir(dir2);
        continue;
      }
      // Check file
      if ( 0 != is_regularfile(laTempDir,lpdirent2->d_name)) {
        lpdirent2 = readdir(dir2);
        continue;
      }
      // Process file
      if (loiVerbose>1) printf("Found pci: %s \n", lpdirent2->d_name );
      strcpy( laTempFile, laTempDir );
      strcat( laTempFile, "/" );
      strcat( laTempFile, lpdirent2->d_name );
      // Read
      fd = fopen( laTempFile, "r" );
      assert(fd!=NULL);
      fread( &buffer, 1, 256, fd );
      fclose(fd);
      // Allocate
      pciitem = chain_append(pc->pci);
      mydevice = calloc(1, sizeof(TPciDevice));
      assert(mydevice!=NULL);
      pciitem->data = mydevice;
      // Fill in
      mydevice->vendor   = getword(buffer+ PCI_VENDOR_ID);
      mydevice->device   = getword(buffer+ PCI_DEVICE_ID);
      mydevice->class    = buffer[PCI_CLASS_DEVICE+1];
      mydevice->subclass = buffer[PCI_CLASS_DEVICE];
      if (loiVerbose>0) printf("Device ..: %4x.%4x  %i.%i \n",
        mydevice->vendor, mydevice->device, mydevice->class, mydevice->subclass
      );
      // next
      lpdirent2 = readdir(dir2);
    }
    // Go On
    lpdirent1 = readdir(dir1);
  }
  return 0;  
}
